# loops_practice_test.py
# L. Lee (LJL2), T. Sarver (TCS95)
# March 19, 2022

import loops_practice as loops
import cornellasserts as ca
import copy

def test_find_max():
    testcases = [
        # format: (input, output)
        ([0], 0),
        ([4,0,12], 12),
        ([-4,-10,-2], -2)
        ]

    for item in testcases:
        (inlist, expected) = item
        origlist = copy.deepcopy(inlist)

        error_msg_start = "find_max problem " # prepare for any errors
        error_msg_start += "on input " + str(repr(inlist)) + ": "

        result = loops.find_max(inlist)

        # didn't change original list
        assert inlist == origlist, \
            error_msg_start + "input list was changed! It's now " + repr(inlist)

        # check values are correct
        assert result == expected, \
            error_msg_start + "expected " + repr(expected) \
            + " but got " + repr(result)

def test_clamp():
        testcases = {((-1, 1, 3, 5), 0, 4): [0, 1, 3, 4],
                 ((1, 3), 0, 4): [1,3],
                 ((-1, 1, 3, 5), 1, 1): [1, 1, 1, 1],
                 ((5, 0, 2, 7, 4), 0, 4.5):[4.5, 0, 2, 4.5, 4],
                 ((5, 0, 1, 7, 4), 0.5, 4): [4, 0.5, 1, 4, 4],
                 ((), 0, 4): []
                 }

        for tc in testcases:
            thelist = list(tc[0])
            m1 = tc[1]
            m2 = tc[2]
            expected = testcases[tc]

            msg = "\ttesting list " + str(thelist) + ", floor: " + str(m1)
            msg += ", " + " ceiling: " + str(m2)
            print(msg)

            returned = clamp_module.clamp(thelist, m1, m2)
            assert returned is None, \
                "Problem: clamp() shouldn't return anything, but it returned "+ repr(returned)

            assert len(thelist) == len(expected), \
                "Problem: clamp() changed the length of the list"

            for i in range(len(expected)):
                assert expected[i] == thelist[i], \
                    "Problem with item at index i. Was " +repr(thelist[i]) + " but should have been " + repr(expected[i])

def test_skipmult():
    testcases = [
        # format: ((input list, input n, input k), what list gets changed to)
        (([1, 3, 5, 7], 2, 2 ), [2, 3, 10, 7]),
        (([1, 3, 5, 7], 2, 3), [2, 3, 5, 14]),
        ]

    for item in testcases:
        theinput = item[0]
        inlist = theinput[0]
        n = theinput[1]
        k = theinput[2]

        correct_change = item[1]
        error_msg_start = "skipmult problem " # prepare for any errors
        error_msg_start += "on input " + str(repr(theinput)) + ": "

        result = loops.skipmult(inlist, n, k)

        # shouldn't return anything
        assert result is None, \
            error_msg_start + "should return None, but returned " + repr(result)

        # check values are correct
        assert inlist == correct_change, \
            error_msg_start + "expected the list to become " + repr(correct_change) \
            + " but it became/stayed " + repr(inlist)


def test_nullify_dups():
    # empty table
    expectedtable1 = [[]]
    table1 = [[]]
    loops.nullify_dups(table1)
    for row_num in range(len(table1)):
        for col_num in range(len(table1[row_num])):
            assert expectedtable1[row_num][col_num] == table1[row_num][col_num]

    # more than one repeat
    table2 = [[1,2,3],[1,2,3]]
    expectedtable2 = [[1,2,3],[0,0,0]]
    loops.nullify_dups(table2)
    for row_num in range(len(table2)):
        for col_num in range(len(table2[row_num])):
            assert expectedtable2[row_num][col_num] == table2[row_num][col_num]

    # no repeats
    table3 = [[1,2,3],[4,5,6]]
    expectedtable3 = [[1,2,3],[4,5,6]]
    loops.nullify_dups(table3)
    for row_num in range(len(table3)):
        for col_num in range(len(table3[row_num])):
            assert expectedtable3[row_num][col_num] == table3[row_num][col_num]

    # one repeat
    table4 = [[1,2,3],[1,5,6]]
    expectedtable4 = [[1,2,3],[0,5,6]]
    loops.nullify_dups(table4)
    for row_num in range(len(table4)):
        for col_num in range(len(table4[row_num])):
            assert expectedtable4[row_num][col_num] == table4[row_num][col_num]

    # table is not square
    table5 = [[1],[4,1],[4,4,5],[5]]
    expectedtable5 = [[1],[4,0],[0,0,5],[0]]
    loops.nullify_dups(table5)
    for row_num in range(len(table5)):
        for col_num in range(len(table5[row_num])):
            assert expectedtable5[row_num][col_num] == table5[row_num][col_num]


def test_encode():
    testcases = [
        ({'a':'o','b':'z'},  'bat', 'zot'),
        ({'a':'o','z':'b'}, 'razzle', 'robble'),
        ({'a':'o','z':'b'}, '', '')
    ]

    for item in testcases:
        cipher = item[0]
        text = item[1]
        expected = item[2]
        orig_cipher = dict(cipher)
        error_msg_start = "encode problem " # prepare for any errors
        error_msg_start += "on input " + str((str(cipher), text)) + ": "

        result = loops.encode(cipher, text)

        # changed cipher
        assert cipher == orig_cipher, \
            error_msg_start + "input cipher was changed! It's now " + repr(cipher)

        # check values are correct
        assert result == expected, \
            error_msg_start + "expected " + repr(expected) \
            + " but got " + repr(result)


print("Now testing loops_practice.py")
print()
print("Testing find_max")
test_find_max()
print("Finished testing find_max")
print()
print("Testing clamp")
test_clamp
print("Finished testing clamp")
print()
print("Testing skipmult")
test_skipmult()
print("Finished testing skipmult")
print()
print("Testing nullify_dups")
test_nullify_dups()
print("Finished testing nullify_dups")
print()
print("Testing encode")
test_encode()
print("Finished testing encode")
print()

print("All your functions passed! You finished this for-loops training!")
